# NimbleCSV

NimbleCSV is a simple and extremely fast library CSV parsing/dumping library for Elixir. Its performance comes from allowing developers to define custom parsers/dumpers which rely on binary patterns for efficiency:

```elixir
# Define the parser
NimbleCSV.define(MyParser, separator: "\t", escape: "\"")

# Parse the data
MyParser.parse_string "name\tage\njohn\t27"
#=> [["john","27"]]
```

NimbleCSV provides both eager and lazy (streaming) parsing as well as data dumping. The library was designed to be simple and compose well. For example, instead of trying to tackle how data is cast or how headers are managed, it allows the developers to build on top of the parsed results:

```elixir
# Lazily parses a file stream
"path/to/file"
|> MyParser.parse_stream
|> Stream.map(fn [name, age] ->
  %{name: name, age: String.to_integer(age)}
end)
```

By default this library ships with `NimbleCSV.RFC4180`, which is the most common implementation of CSV parsing/dumping available using comma as separators and double-quote as escape. If you want to use it in your codebase, simply alias it to CSV and enjoy:

```elixir
alias NimbleCSV.RFC4180, as: CSV
CSV.parse_string "name,age\njohn,27"
[["john","27"]]
```

See the [online documentation](https://hexdocs.pm/nimble_csv) for more information.

## Installation

  1. Add `nimble_csv` to your list of dependencies in `mix.exs`:

    ```elixir
    def deps do
      [{:nimble_csv, "~> 0.1.0"}]
    end
    ```

  2. Ensure `nimble_csv` is started before your application:

    ```elixir
    def application do
      [applications: [:nimble_csv]]
    end
    ```

## License

Copyright 2016 Plataformatec

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.